/* boot_verified-boot.h - definitions used in boot_verified-boot.c */
/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (C) 1999,2000,2001,2002,2003,2004  Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GRUB_BOOT_VERIFIED_BOOT_H__
#define __GRUB_BOOT_VERIFIED_BOOT_H__

#include "boot_rsa-sha.h"

#define SLEEP_MULTI_FACTOR              20
/*
 * name of DER constants
 * These constants comply with DER encoded the ANS1 type tags.
 * DER encoding uses hexadecimal representation.
 * An example DER sequence is:\n
 * - 0x02 -- tag indicating INTEGER
 * - 0x01 -- length in octets
 * - 0x05 -- value
 * Such sequences are typically read into ::x509_buf.
 */
#define ASN1_BOOLEAN                    0x01
#define ASN1_INTEGER                    0x02
#define ASN1_BIT_STRING                 0x03
#define ASN1_OCTET_STRING               0x04
#define ASN1_NULL                       0x05
#define ASN1_OID                        0x06
#define ASN1_UTF8_STRING                0x0C
#define ASN1_SEQUENCE                   0x10
#define ASN1_SET                        0x11
#define ASN1_PRINTABLE_STRING           0x13
#define ASN1_T61_STRING                 0x14
#define ASN1_IA5_STRING                 0x16
#define ASN1_UTC_TIME                   0x17
#define ASN1_GENERALIZED_TIME           0x18
#define ASN1_UNIVERSAL_STRING           0x1C
#define ASN1_BMP_STRING                 0x1E
#define ASN1_PRIMITIVE                  0x00
#define ASN1_CONSTRUCTED                0x20
#define ASN1_CONTEXT_SPECIFIC           0x80

#define OID_PKCS1                       "\x2A\x86\x48\x86\xF7\x0D\x01\01"
#define OID_PKCS1_RSA                   OID_PKCS1"\x01"
#define OID_PKCS1_SHA256                OID_PKCS1"\x05"

#define SIZE_BZIMAGE_SIGN               4096
#define SIZE_CERT_PEM                   4096
#define SIZE_CERT_ASN1                  2048
#define SIZE_RAS_SIGN_HEX               256 /*Max length of rsa signature length*/
#define SIZE_SHA256_HASH_HEX            32 /*Max length of sha256 hash hex format*/
#define SIZE_X509_NAME_MAX              64 /*Max lengt of x509 subj */

#define ERR_GROUP_BASE64                0
#define ERR_BASE64_INVALID_CHARACTER    (ERR_GROUP_BASE64 + 1)
#define ERR_BASE64_BUFFER_TOO_SMALL     (ERR_GROUP_BASE64 + 2)

#define ERR_GROUP_PEM                   100
#define ERR_PEM_NO_BEGIN                (ERR_GROUP_PEM + 1)
#define ERR_PEM_NO_END                  (ERR_GROUP_PEM + 2)
#define ERR_PEM_CHARACTER               (ERR_GROUP_PEM + 3)
#define ERR_PEM_INDEX_POSITION          (ERR_GROUP_PEM + 4)

#define ERR_GROUP_ASN1                  200
#define ERR_ASN1_OUT_OF_DATA            -(ERR_GROUP_ASN1 + 1)
#define ERR_ASN1_UNEXPECTED_TAG         -(ERR_GROUP_ASN1 + 2)
#define ERR_ASN1_INVALID_LENGTH         -(ERR_GROUP_ASN1 + 3)
#define ERR_ASN1_LENGTH_MISMATCH        -(ERR_GROUP_ASN1 + 4)

#define ERR_GROUP_X509                  100000
#define ERR_X509_CERT_INVALID_INPUT     (ERR_GROUP_X509 + 1000)
#define ERR_X509_CERT_INVALID_FORMAT    (ERR_GROUP_X509 + 2000)
#define ERR_X509_CERT_INVALID_TOP_LEN   (ERR_GROUP_X509 + 3000)
#define ERR_X509_CERT_INVALID_VERSION   (ERR_GROUP_X509 + 4000)
#define ERR_X509_CERT_UNKNOWN_VERSION   (ERR_GROUP_X509 + 5000)
#define ERR_X509_CERT_INVALID_SERIAL    (ERR_GROUP_X509 + 6000)
#define ERR_X509_CERT_INVALID_ALG       (ERR_GROUP_X509 + 7000)
#define ERR_X509_CERT_INVALID_NAME      (ERR_GROUP_X509 + 8000)
#define ERR_X509_NAME_SPACE_FULL        (ERR_GROUP_X509 + 9000)
#define ERR_X509_CERT_INVALID_DATE      (ERR_GROUP_X509 + 10000)
#define ERR_X509_CERT_INVALID_SIGNATURE (ERR_GROUP_X509 + 11000)
#define ERR_X509_CERT_UNSUPPORT_PKALG   (ERR_GROUP_X509 + 12000)
#define ERR_X509_CERT_INVALID_PUBKEY    (ERR_GROUP_X509 + 13000)

#define ERR_GROUP_VERIFYBOOT            200000
#define ERR_VERIFYBOOT_OPEN_FAIL        (ERR_GROUP_VERIFYBOOT + 1000)
#define ERR_VERIFYBOOT_READ_FAIL        (ERR_GROUP_VERIFYBOOT + 2000)
#define ERR_VERIFYBOOT_MAGIC_NUM        (ERR_GROUP_VERIFYBOOT + 3000)
#define ERR_VERIFYBOOT_SIGN_LEN         (ERR_GROUP_VERIFYBOOT + 4000)
#define ERR_VERIFYBOOT_LEN_MISMATCH     (ERR_GROUP_VERIFYBOOT + 5000)
#define ERR_VERIFYBOOT_LOAD_CERT        (ERR_GROUP_VERIFYBOOT + 6000)
#define ERR_VERIFYBOOT_CERT_HASH        (ERR_GROUP_VERIFYBOOT + 7000)

#define CNT_SUBJ_NAME                   7 /*C,ST,L,O,OU,CN,email*/

#ifdef  DEBUG_VERIFIED_BOOT
#define GRUB_ERR(fmt, arg...)           grub_printf("ERROR %s[%d]: "fmt, __FUNCTION__, __LINE__, ##arg)
#define GRUB_INFO(fmt, arg...)          grub_printf("[GRUB INFO]: "fmt, ##arg)
#define GRUB_DBG(fmt, arg...)           grub_printf("%s[%d]: "fmt, __FUNCTION__, __LINE__, ##arg)
#define GRUB_DBG_WAIT()                  count_wait(60)
#else
#define GRUB_ERR(fmt, arg...)
#define GRUB_INFO(fmt, arg...)
#define GRUB_DBG(fmt, arg...)
#define GRUB_DBG_WAIT()
#endif

typedef struct {
    unsigned char *buf;
    int len;
} x509_data_buf;

typedef struct _x509_name {
    x509_data_buf val;
    struct _x509_name *next;
} x509_name;

typedef struct {
    int year, mon, day;
    int hour, min, sec;
} x509_time;

typedef struct {
    x509_data_buf pubAlg;                       /* publick key OID_PKCS1_RSA */
    rsa_context   rsa;                          /* public key*/
} x509_pubkey;

typedef struct {
    int version;                                /* version number 1-3 */
    x509_data_buf tbs;                          /* TBSCertificate{buf,len}*/
    x509_data_buf sign;                         /* sha256 hash signature */
    x509_data_buf serial;                       /* serial number */
    x509_data_buf tbsAlg;                       /* alg  */
    x509_data_buf signAlg;                      /* alg  */
    x509_name     issuer[CNT_SUBJ_NAME];        /* Issuer: C,ST,L,O,OU,CN,email*/
    x509_name     subject[CNT_SUBJ_NAME];       /* Subject: C,ST,L,O,OU,CN,email*/
    x509_time     valid_from;                   /* Validity from*/
    x509_time     valid_to;                     /* Validity from*/
    x509_pubkey   pubkey;
} x509_cert;

typedef struct {
    char          mver;                         /* bzImage main version */
    char          sver;                         /* bzImage sub version */
    char          ownerid;                      /* bzImage owner ID */
    char          vendorid;                     /* bzImage vendor ID */
    char          timestamp[2];                 /* bzImage time stamp */
    char          algid;                        /* bzImage alog RAS_ID */
    char          hashid;                       /* bzImage hash ID */
} image_header;

typedef struct {
    unsigned char *filename;                    /* bzImage file name */
    int           filelen;                      /* bzImage file len */
    unsigned char *signbuf;                     /* bzImage signature data */
    int           signlen;                      /* bzImage signature len */
    unsigned char *hashbuf;                     /* sha256 hash of image+header */
    int           hashlen;                      /* sha256 hash len */
    unsigned char *certbuf;                     /* certificate in signature */
    int           certlen;                      /* certificate file len */
    image_header  header;                       /* bzImage signature header */
} image_sign;

#endif /* #ifndef __GRUB_BOOT_VERIFIED_BOOT_H__ */
